import machine, bluetooth, aioble, uasyncio, time, gc, config

LOOPTIME = 6

CURRENT_TIME_SERVICE_UUID = bluetooth.UUID(0x1805)
CURRENT_TIME_CHARACTERISTIC_UUID = bluetooth.UUID(0x2A2B)

# https://github.com/RideBeeline/android-bluetooth-current-time-service
# https://github.com/androidthings/sample-bluetooth-le-gattserver
# https://docs.micropython.org/en/latest/library/bluetooth.html
# https://docs.espressif.com/projects/esp-idf/en/latest/esp32/api-guides/blufi.html
# https://docs.espressif.com/projects/esp-idf/en/v5.0/esp32c3/api-guides/coexist.html
# https://github.com/micropython/micropython-lib/tree/master/micropython/bluetooth/aioble
# https://docs.espressif.com/projects/esp-idf/en/latest/esp32/api-guides/coexist.html
# /home/dworz/dworzak.ch/054_Word_Clock/git/micropython/tools/mpremote/mpremote.py connect /dev/ttyACM1 mip install aioble-central aioble-client

valid = False
async def advertise():
	while True:
		if config.Radio == 'Bluetooth':
			try:
				async with await aioble.advertise(100, name=config.Hostname) as connection:
					print("Connection from", connection.device)
					await connection.disconnected()
			except Exception as e:
				print(f'cts advertise Exception {e}')
				await uasyncio.sleep(10*LOOPTIME)
		else: await uasyncio.sleep(LOOPTIME)

async def current_time_service():
	global valid

# 	print('current_time_service')
	rtc = machine.RTC()
	start = time.time()
	while True:
		gc.collect()
		await uasyncio.sleep(LOOPTIME)
		if config.Radio == 'Bluetooth':
			if valid and (time.time() - start) < (config.TimeInterval * 60): continue
			print(f'cts {aioble.core.ble.active()=}')
			aioble.core.ensure_active()
			devices = []
			try:
				async with aioble.scan(duration_ms=2000, interval_us=30000, window_us=30000) as scanner:
					async for result in scanner:
						if CURRENT_TIME_SERVICE_UUID in result.services() and result.device not in devices: devices.append(result.device)
			except Exception as e: print(f'cts {e}')
			print(f'cts {[device.addr_hex() for device in devices]}')
			for device in devices:
				try:
					print(f'cts > {device.addr_hex()}')
					async with await device.connect() as connection:
						current_time_service = await connection.service(CURRENT_TIME_SERVICE_UUID)
						current_time_characteristic = await current_time_service.characteristic(CURRENT_TIME_CHARACTERISTIC_UUID)
						data = await current_time_characteristic.read()
						rtc.datetime((int.from_bytes(data[0:2], 'little'), data[2], data[3], data[7] + 1, data[4], data[5], data[6], 0))	# year, month, day, weekday, hours, minutes, seconds, subseconds
					valid, start = True, time.time()
					dt = time.localtime()
					print(f'cts < {dt[0]:04d}.{dt[1]:02d}.{dt[2]:02d} {dt[3]:02d}:{dt[4]:02d}:{dt[5]:02d}')
					break
				except Exception as e: print(f'cts Exception {e}')
			else: print('cts no devices')
		else: aioble.stop()

async def main():
	await uasyncio.gather(current_time_service(), advertise())

if __name__ == '__main__':
	config.TimeInterval = 0.1
	uasyncio.run(main())
